using System;
using System.Collections.Generic;
using System.IO;
using System.Net;
using System.Text.RegularExpressions;
using System.Windows.Forms;
using KellermanSoftware.NetFtpLibrary;

namespace KellermanSoftware.NetFtpLibraryDemo
{
    public partial class Demo : Form
    {
        private FTP _ftp;
        SortedList<string, int> timeZones;
        string _CurrentLocalDirectory;
        bool _Loading = false;

        public Demo()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Get a remote directory listing and display it in the list box
        /// </summary>
        /// <param name="dirName">The sub directory to load, string.empty to reload the current directory</param>
        private void loadRemoteDirectory(String dirName)
        {
            if (_Loading == true)
            {
                return;
            }

            _Loading = true;
            this.Cursor = Cursors.WaitCursor;

            //string myString = _ftp.GetRawDirectoryListing();

            if (dirName == "..")
            {
                _ftp.CurrentDirectory = _ftp.ParentDirectory;
            }
            else
            {
                _ftp.CurrentDirectory += "/" + dirName;
            }

            while (_ftp.IsBusy)
            {
                System.Threading.Thread.Sleep(100);
            }

            List<FTPFileInfo> files;
            try
            {
                files = _ftp.GetDirectoryListing();
            }
            catch (WebException ex)
            {
                MessageBox.Show("Web error:" + ex.Message);
                return;
            }

            listRemote.Items.Clear();
            ListViewItem lvi = new ListViewItem("..");
            lvi.ImageIndex = 1;
            listRemote.Items.Add(lvi);

            foreach (FTPFileInfo file in files)
            {
                if (file.IsDirectory)
                {
                    if (file.FileName != ".." && file.FileName != ".")
                    {
                        lvi = new ListViewItem(file.FileName);
                        lvi.ImageIndex= 0;
                        listRemote.Items.Add(lvi);
                    }
                }
            }

            foreach (FTPFileInfo file in files)
            {
                if (!file.IsDirectory)
                {
                    lvi = new ListViewItem(file.FileName);
                    lvi.SubItems.Add(file.SizeInBytes.ToString());
                    lvi.SubItems.Add(file.Modified.ToString());
                    listRemote.Items.Add(lvi);
                }
            }

            if(_ftp.CurrentDirectory != string.Empty && !cmboRemotePath.Items.Contains('/' + _ftp.CurrentDirectory))
                cmboRemotePath.Items.Add(_ftp.CurrentDirectory);
            cmboRemotePath.SelectedItem = _ftp.CurrentDirectory;

            this.Cursor = Cursors.Default;
            _Loading = false;
        }

        /// <summary>
        /// Property to store the current path of the display local directory
        /// </summary>
        private string CurrentLocalDirectory
        {
            get 
            {
                return _CurrentLocalDirectory;
            }
            set 
            {
                if (!value.EndsWith(new String(Path.DirectorySeparatorChar, 1)))
                    value += Path.DirectorySeparatorChar;
                _CurrentLocalDirectory = value;
            }
        }

        /// <summary>
        /// Get a local directory listing and display it in the list box
        /// </summary>
        /// <param name="dirName">The sub directory to load, string.empty to reload the current directory</param>
        private void loadLocalDirectory(String dirName)
        {
            if (dirName == "..")
            {
                if (CurrentLocalDirectory.Length > 1)
                {
                    try
                    {
                        DirectoryInfo d = Directory.GetParent(CurrentLocalDirectory);
                        if (d != null)
                            CurrentLocalDirectory = d.Parent.FullName;
                    }
                    catch (WebException ex)
                    {
                        MessageBox.Show("Web error:" + ex.Message);
                        return;
                    }
                }
            }
            else
                CurrentLocalDirectory += dirName;

            
            listLocal.Items.Clear();
            ListViewItem lvi = new ListViewItem("..");
            lvi.ImageIndex = 1;
            listLocal.Items.Add(lvi);

            string[] dirs = Directory.GetDirectories(CurrentLocalDirectory);
            foreach (String dir in dirs)
            {
                DirectoryInfo d = new DirectoryInfo(dir);
                lvi = new ListViewItem(d.Name);
                lvi.ImageIndex = 0;
                listLocal.Items.Add(lvi);
            }

            DirectoryInfo curDir = new DirectoryInfo(CurrentLocalDirectory);
            foreach (FileInfo file in curDir.GetFiles())
            {
                lvi = new ListViewItem(file.Name);
                lvi.SubItems.Add(file.Length.ToString());
                lvi.SubItems.Add(file.LastWriteTimeUtc.ToString());
                listLocal.Items.Add(lvi);
            }
        }

        /// <summary>
        /// Creates a WebProxy from the user entered information
        /// </summary>
        /// <returns>WebProxy from user inputs</returns>
        private WebProxy getProxyFromGUI()
        {
            WebProxy proxy = new WebProxy();
            if (txtProxyHost.Text.Length == 0)
                return proxy;
            proxy.Address = new Uri(txtProxyHost.Text);
            proxy.Credentials = new NetworkCredential(txtProxyUser.Text, txtProxyPass.Text);
            return proxy;
        }
         
        /// <summary>
        /// Connects to the FTP server
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnConnect_Click(object sender, EventArgs e)
        {
            this.Cursor = Cursors.WaitCursor;
            if (groupLogin.Enabled)
            {                
                _ftp.EnableSSL = chkSSL.Checked;
                _ftp.HostAddress = txtHost.Text;
                _ftp.OverwriteFiles = chkOverWrite.Checked;
                _ftp.OverwriteReadOnlyFiles = chkReadOnly.Checked;
                _ftp.Password = txtPass.Text;
                _ftp.Port = Int32.Parse(txtPort.Text);
                _ftp.ReadWriteTimeout = Int32.Parse(txtReadWriteTimeout.Text);
                _ftp.Timeout = Int32.Parse(txtTimeout.Text);
                _ftp.UseBinary = chkBinary.Checked;
                _ftp.UsePassive = chkPassive.Checked;
                _ftp.UserName = txtLogin.Text;
                WebProxy proxy = getProxyFromGUI();

                if (txtFileLog.Text.Length > 0)
                    _ftp.EnableLogging(txtFileLog.Text);

                try
                {
                    _ftp.Connect();
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message, "FTP Connect Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    this.Cursor = Cursors.Default;
                    return;
                }

                groupLogin.Enabled = false;
                groupOptions.Enabled = false;
                btnConnect.Text = "Disconnect";
                btnConnect.ImageKey = "disconnect";
                splitContainer1.Enabled = true;

                try
                {
                    loadRemoteDirectory(string.Empty);
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message, "FTP Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
            }
            else
            {
                _ftp.Disconnect();
                _ftp.Dispose();
                groupLogin.Enabled = true;
                groupOptions.Enabled = true;
                btnConnect.ImageKey = "connect";
                btnConnect.Text = "Connect";
                splitContainer1.Enabled = false;
            }
            this.Cursor = Cursors.Default;
        }

        /// <summary>
        /// Fired when the form is loaded, initializes the FTP component and the GUI
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Form1_Load(object sender, EventArgs e)
        {
            try
            {
                _ftp = new FTP();
                _ftp.StrictDirectoryParsing = false;

                // load the timezone list
                timeZones = _ftp.TimeZoneList;
                foreach (string timeZoneName in timeZones.Keys)
                    cmboGMT.Items.Add(timeZoneName);

                txtTimeout.Text = _ftp.Timeout.ToString();
                txtReadWriteTimeout.Text = _ftp.ReadWriteTimeout.ToString();

                // load the drive list box
                foreach (DriveInfo drive in DriveInfo.GetDrives())
                    if (drive.Name == @"C:\")
                    {
                        int i = cmboDrive.Items.Add(drive.Name);
                        cmboDrive.SelectedIndex = i;
                    }
                    else
                        cmboDrive.Items.Add(drive.Name);


                // load the local directory
                CurrentLocalDirectory = @"C:\";
                loadLocalDirectory(string.Empty);

                //Get defaults from registry
                txtHost.Text= GetReg("KellermanSoftware", "Kellerman FTP", "Host");
                txtLogin.Text =GetReg("KellermanSoftware", "Kellerman FTP", "Login");
            }
            catch (Exception myException)
            {
                MessageBox.Show(myException.Message);
                return;
            }
        }

        /// <summary>
        /// Uploads selected file(s) and directory(ies)
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnUpload_Click(object sender, EventArgs e)
        {
            try
            {
                for (int i = 0; i < listLocal.SelectedItems.Count; i++)
                {
                    if (listLocal.SelectedItems[i].ImageIndex == 1)
                        continue;

                    if (listLocal.SelectedItems[i].ImageIndex == -1)
                    {
                        ProgressForm statusDialog;

                        if (_ftp.FileExists(listLocal.SelectedItems[i].Text))
                        {
                            if (MessageBox.Show("File already exists! If you cancelled the upload and would like to resume it, press yes. If you did not cancel the upload, press no.", "Resume Upload", MessageBoxButtons.YesNo) == DialogResult.Yes)
                            {
                                using (statusDialog = new ProgressForm())
                                {
                                    statusDialog._ftp = _ftp;
                                    statusDialog.isDownloading = false;
                                    _ftp.FileProgressEvent += new FTP.TransferProgressChangedEventHandler(statusDialog.ftp_ProgressEvent);
                                    _ftp.TransferCompleteEvent += new FTP.TransferCompleteEventHandler(statusDialog.ftp_TransferCompleteEvent);
                                    _ftp.ResumeUploadFileAsync(CurrentLocalDirectory + listLocal.SelectedItems[i].Text, _ftp.CurrentDirectory + "/" + listLocal.SelectedItems[i].Text);
                                    statusDialog.ShowDialog();
                                    _ftp.FileProgressEvent -= new FTP.TransferProgressChangedEventHandler(statusDialog.ftp_ProgressEvent);
                                    _ftp.TransferCompleteEvent -= new FTP.TransferCompleteEventHandler(statusDialog.ftp_TransferCompleteEvent);
                                }
                                loadRemoteDirectory(string.Empty);
                            }
                            return;
                        }

                        using (statusDialog = new ProgressForm())
                        {
                            statusDialog._ftp = _ftp;
                            statusDialog.isDownloading = false;
                            _ftp.FileProgressEvent += new FTP.TransferProgressChangedEventHandler(statusDialog.ftp_ProgressEvent);
                            _ftp.TransferCompleteEvent += new FTP.TransferCompleteEventHandler(statusDialog.ftp_TransferCompleteEvent);
                            _ftp.UploadFileAsync(CurrentLocalDirectory + listLocal.SelectedItems[i].Text, _ftp.CurrentDirectory + "/" + listLocal.SelectedItems[i].Text);
                            statusDialog.ShowDialog();
                            _ftp.FileProgressEvent -= new FTP.TransferProgressChangedEventHandler(statusDialog.ftp_ProgressEvent);
                            _ftp.TransferCompleteEvent -= new FTP.TransferCompleteEventHandler(statusDialog.ftp_TransferCompleteEvent);
                        }
                        loadRemoteDirectory(string.Empty);
                    }
                    else
                    {
                        using (ProgressForm statusDialog = new ProgressForm())
                        {
                            statusDialog._ftp = _ftp;
                            statusDialog.isDownloading = false;
                            _ftp.FileProgressEvent += new FTP.TransferProgressChangedEventHandler(statusDialog.ftp_ProgressEvent);
                            _ftp.TransferCompleteEvent += new FTP.TransferCompleteEventHandler(statusDialog.ftp_TransferCompleteEvent);

                            if (optNormal.Checked == true)
                                _ftp.UploadDirectoryAsync(CurrentLocalDirectory + listLocal.SelectedItems[i].Text, listLocal.SelectedItems[i].Text);
                            else if (optPattern.Checked == true)
                                _ftp.UploadDirectoryAsync(CurrentLocalDirectory + listLocal.SelectedItems[i].Text, listLocal.SelectedItems[i].Text, true, txtFilter.Text);
                            else if (optRegex.Checked == true)
                                _ftp.UploadDirectoryAsync(CurrentLocalDirectory + listLocal.SelectedItems[i].Text, listLocal.SelectedItems[i].Text, true, new Regex(txtFilter.Text));
                            else if (optSize.Checked == true)
                                _ftp.UploadDirectoryAsync(listLocal.SelectedItems[i].Text, CurrentLocalDirectory + listLocal.SelectedItems[i].Text, true, Int32.Parse(txtFilter.Text), Int32.Parse(txtFilter2.Text));
                            else if (optDate.Checked == true)
                                _ftp.UploadDirectoryAsync(listLocal.SelectedItems[i].Text, CurrentLocalDirectory + listLocal.SelectedItems[i].Text, true, DateTime.Parse(txtFilter.Text), DateTime.Parse(txtFilter2.Text));

                            statusDialog.ShowDialog();
                            _ftp.FileProgressEvent -= new FTP.TransferProgressChangedEventHandler(statusDialog.ftp_ProgressEvent);
                            _ftp.TransferCompleteEvent -= new FTP.TransferCompleteEventHandler(statusDialog.ftp_TransferCompleteEvent);
                        }
                        loadRemoteDirectory(string.Empty);
                    }
                }
            }
            catch (WebException ex)
            {
                MessageBox.Show("Web error:" + ex.Message);
                return;
            }
        }

        /// <summary>
        /// If a folder, or the up folder icon, is double clicked it is navigated to
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void listRemote_DoubleClick(object sender, System.EventArgs e)
        {
            try
            {
                if (listRemote.SelectedItems.Count == 0 || listRemote.SelectedItems[0].ImageIndex == -1)
                    return;
                loadRemoteDirectory(listRemote.SelectedItems[0].Text);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        /// <summary>
        /// Creates a remote folder
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnRemoteNewFolder_Click(object sender, EventArgs e)
        {
            listRemote.LabelEdit = true;
            ListViewItem lvi = listRemote.Items.Add("New Folder");
            lvi.ImageIndex = 0;
            lvi.BeginEdit();     
        }

        /// <summary>
        /// Deletes the selected remote file/folder
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnRemoteDelete_Click(object sender, EventArgs e)
        {
            if (listRemote.SelectedItems.Count == 0 || listRemote.SelectedItems[0].Text == "..")
                return;
            try
            {
                if (listRemote.SelectedItems[0].ImageIndex == 0)
                {
                    _ftp.DeleteDirectory(listRemote.SelectedItems[0].Text);
                }
                else
                {
                    _ftp.DeleteFile(listRemote.SelectedItems[0].Text);
                }
            }
            catch (WebException ex)
            {
                MessageBox.Show("Web error:" + ex.Message);
                return;
            }
            loadRemoteDirectory(string.Empty);
        }

        /// <summary>
        /// Refreshes the remote directory listing
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnRemoteRefresh_Click(object sender, EventArgs e)
        {
            loadRemoteDirectory(string.Empty);
        }

        /// <summary>
        /// Fired after the user is done entering a new/renaming a folder and file
        /// It validates the name is valid and then updates it
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void listRemote_AfterLabelEdit(object sender, System.Windows.Forms.LabelEditEventArgs e)
        {
            e.CancelEdit = true;
            if (e.Label == null || e.Label.Length == 0)
                return;
            try
            {
                if (listRemote.Items[e.Item].ImageIndex == 0)
                {
                    if (listRemote.Items[e.Item].Text == "New Folder")
                    {
                        if (_ftp.CreateDirectory(e.Label))
                        {
                            e.CancelEdit = false;
                        }
                        else
                        {
                            MessageBox.Show("Directory creation failed.");
                        }
                    }
                    else
                    {
                        if (_ftp.RenameDirectory(listRemote.Items[e.Item].Text, e.Label))
                        {
                            e.CancelEdit = false;
                        }
                        else
                        {
                            MessageBox.Show("Directory rename failed.");
                        }
                    }
                }
                else
                {
                    if (_ftp.RenameFile(listRemote.Items[e.Item].Text, e.Label))
                    {
                        e.CancelEdit = false;
                    }
                    else
                    {
                        MessageBox.Show("File rename failed.");
                    }
                }
            }
            catch (WebException ex)
            {
                MessageBox.Show("Web error:" + ex.Message);
                return;
            }
            catch (Exception)
            {
                return;
            }
			
            loadRemoteDirectory(string.Empty);
        }

        /// <summary>
        /// Renames the selected folder/file
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnRemoteRename_Click(object sender, EventArgs e)
        {
            if (listRemote.SelectedItems.Count == 0 || listRemote.SelectedItems[0].Text == "..")
                return;

            listRemote.LabelEdit = true;
            listRemote.SelectedItems[0].BeginEdit();
        }

        /// <summary>
        /// Adjusts the filter GUI to disable both textboxes
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void optNormal_CheckedChanged(object sender, EventArgs e)
        {
            if (optNormal.Checked)
            {
                txtFilter.Enabled = false;
                txtFilter2.Enabled = false;
                txtFilter.Text = "";
                txtFilter2.Text = "";
            }
        }

        /// <summary>
        /// Adjusts the filter GUI to show one text box
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void optPattern_CheckedChanged(object sender, EventArgs e)
        {
            if (optPattern.Checked)
            {
                txtFilter.Enabled = true;
                txtFilter2.Enabled = false;
                txtFilter.Text = "";
                txtFilter2.Text = "";
            }
        }

        /// <summary>
        /// Adjusts the filter GUI to show one text box
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void optRegex_CheckedChanged(object sender, EventArgs e)
        {
            if (optRegex.Checked)
            {
                txtFilter.Enabled = true;
                txtFilter2.Enabled = false;
                txtFilter.Text = "";
                txtFilter2.Text = "";
            }
        }

        /// <summary>
        /// Adjusts the filter GUI to show both text boxes
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void optSize_CheckedChanged(object sender, EventArgs e)
        {
            if (optSize.Checked)
            {
                txtFilter.Enabled = true;
                txtFilter2.Enabled = true;
                txtFilter.Text = "";
                txtFilter2.Text = "";
            }
        }

        /// <summary>
        /// Adjusts the filter GUI to show both text boxes
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void optDate_CheckedChanged(object sender, EventArgs e)
        {
            if (optDate.Checked)
            {
                txtFilter.Enabled = true;
                txtFilter2.Enabled = true;
                txtFilter.Text = "";
                txtFilter2.Text = "";
            }
        }

        /// <summary>
        /// If a folder, or the up folder icon, is double clicked it is navigated to
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void listLocal_DoubleClick(object sender, System.EventArgs e)
        {
            if (listLocal.SelectedItems.Count == 0 || listLocal.SelectedItems[0].ImageIndex == -1)
                return;
            loadLocalDirectory(listLocal.SelectedItems[0].Text);           
        }

        /// <summary>
        /// Fired after the user is done entering a new/renaming a folder and file
        /// It validates the name is valid and then updates it
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void listLocal_AfterLabelEdit(object sender, System.Windows.Forms.LabelEditEventArgs e)
        {
            e.CancelEdit = true;
            if (e.Label == null || e.Label.Length == 0)
                return;

            if (listLocal.Items[e.Item].ImageIndex == 0)
            {
                if (listLocal.Items[e.Item].Text == "New Folder")
                {
                    try
                    {
                        if (!Directory.Exists(CurrentLocalDirectory + e.Label))
                        {
                            Directory.CreateDirectory(CurrentLocalDirectory + e.Label);
                            e.CancelEdit = false;
                        }else
                            MessageBox.Show("Directory creation failed.");
                    }
                    catch (Exception)
                    {
                        MessageBox.Show("Directory creation failed.");
                    }     
                }
                else
                {
                    try
                    {
                        if (!Directory.Exists(CurrentLocalDirectory + e.Label))
                        {
                            Directory.Move(CurrentLocalDirectory + listLocal.Items[e.Item].Text, CurrentLocalDirectory + e.Label);
                            e.CancelEdit = false;
                        }else
                            MessageBox.Show("Directory rename failed.");
                    }
                    catch (Exception)
                    {
                        MessageBox.Show("Directory rename failed.");
                    }    
                }
            }
            else
            {
                try
                {
                    if (!File.Exists(CurrentLocalDirectory + e.Label))
                    {
                        File.Move(CurrentLocalDirectory + listLocal.Items[e.Item].Text, CurrentLocalDirectory + e.Label);
                        e.CancelEdit = false;
                    }
                    else
                        MessageBox.Show("File rename failed.");
                }
                catch (Exception)
                {
                    MessageBox.Show("File rename failed.");
                }    
            }

            loadLocalDirectory(string.Empty);
        }

        /// <summary>
        /// Creates a new local folder
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnLocalNewFolder_Click(object sender, EventArgs e)
        {
            listLocal.LabelEdit = true;
            ListViewItem lvi = listLocal.Items.Add("New Folder");
            lvi.ImageIndex = 0;
            lvi.BeginEdit();  
        }

        /// <summary>
        /// Deletes the selected folder/file
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnLocalDelete_Click(object sender, EventArgs e)
        {
            listLocal.LabelEdit = false;
            if (listLocal.SelectedItems.Count == 0 || listLocal.SelectedItems[0].Text == "..")
                return;
            try
            {
                string f = CurrentLocalDirectory + listLocal.SelectedItems[0].Text;
                if (listLocal.SelectedItems[0].ImageIndex == 0)
                {
                    Directory.Delete(f, true);
                }
                else
                {
                    File.Delete(f);
                }
            }
            catch (WebException ex)
            {
                MessageBox.Show("Web error:" + ex.Message);
                return;
            }
            
            
            loadLocalDirectory(string.Empty);
        }

        /// <summary>
        /// Refreshes the current local directory
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnLocalRefresh_Click(object sender, EventArgs e)
        {
            loadLocalDirectory(string.Empty);
        }

        /// <summary>
        /// Renames the currently selected file/folder
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnLocalRename_Click(object sender, EventArgs e)
        {
            if (listLocal.SelectedItems.Count == 0 || listLocal.SelectedItems[0].Text == "..")
                return;

            listLocal.LabelEdit = true;
            listLocal.SelectedItems[0].BeginEdit();
        }

        /// <summary>
        /// Downloads the currently selected remote files/folders
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnDownload_Click(object sender, EventArgs e)
        {
            try
            {
                for (int i = 0; i < listRemote.SelectedItems.Count; i++)
                {
                    if (listRemote.SelectedItems[i].ImageIndex == 1)
                        continue;

                    if (listRemote.SelectedItems[i].ImageIndex == -1)
                    {
                        ProgressForm statusDialog;

                        if (File.Exists(CurrentLocalDirectory + listRemote.SelectedItems[i].Text))
                        {
                            if (MessageBox.Show("File already exists! If you cancelled the download and would like to resume it, press yes. If you did not cancel the download, press no.", "Resume Download", MessageBoxButtons.YesNo) == DialogResult.Yes)
                            {
                                statusDialog = new ProgressForm();
                                statusDialog._ftp = _ftp;
                                statusDialog.isDownloading = true;

                                _ftp.FileProgressEvent += new FTP.TransferProgressChangedEventHandler(statusDialog.ftp_ProgressEvent);
                                _ftp.TransferCompleteEvent += new FTP.TransferCompleteEventHandler(statusDialog.ftp_TransferCompleteEvent);
                                _ftp.ResumeDownloadFileAsync(CurrentLocalDirectory + listRemote.SelectedItems[i].Text, listRemote.SelectedItems[i].Text);
                                if (!statusDialog.IsDisposed)
                                    statusDialog.ShowDialog();
                                _ftp.FileProgressEvent -= new FTP.TransferProgressChangedEventHandler(statusDialog.ftp_ProgressEvent);
                                _ftp.TransferCompleteEvent -= new FTP.TransferCompleteEventHandler(statusDialog.ftp_TransferCompleteEvent);
                                loadLocalDirectory(string.Empty);
                            }
                            return;
                        }

                        statusDialog = new ProgressForm();
                        statusDialog._ftp = _ftp;
                        statusDialog.isDownloading = true;

                        _ftp.FileProgressEvent += new FTP.TransferProgressChangedEventHandler(statusDialog.ftp_ProgressEvent);
                        _ftp.TransferCompleteEvent += new FTP.TransferCompleteEventHandler(statusDialog.ftp_TransferCompleteEvent);
                        _ftp.DownloadFileAsync(CurrentLocalDirectory + listRemote.SelectedItems[i].Text, listRemote.SelectedItems[i].Text);
                        if (!statusDialog.IsDisposed)
                            statusDialog.ShowDialog();
                        _ftp.FileProgressEvent -= new FTP.TransferProgressChangedEventHandler(statusDialog.ftp_ProgressEvent);
                        _ftp.TransferCompleteEvent -= new FTP.TransferCompleteEventHandler(statusDialog.ftp_TransferCompleteEvent);
                    }
                    else
                    {
                        ProgressForm statusDialog = new ProgressForm();
                        statusDialog._ftp = _ftp;
                        statusDialog.isDownloading = true;

                        _ftp.OverallProgressEvent += new FTP.TransferProgressChangedEventHandler(statusDialog.ftp_ProgressEvent);
                        _ftp.TransferCompleteEvent += new FTP.TransferCompleteEventHandler(statusDialog.ftp_TransferCompleteEvent);

                        if (optNormal.Checked == true)
                            _ftp.DownloadDirectoryAsync(CurrentLocalDirectory, listRemote.SelectedItems[i].Text);
                        else if (optPattern.Checked == true)
                            _ftp.DownloadDirectoryAsync(CurrentLocalDirectory, listRemote.SelectedItems[i].Text, true, txtFilter.Text);
                        else if (optRegex.Checked == true)
                            _ftp.DownloadDirectoryAsync(CurrentLocalDirectory, listRemote.SelectedItems[i].Text, true, new Regex(txtFilter.Text));
                        else if (optSize.Checked == true)
                            _ftp.DownloadDirectoryAsync(CurrentLocalDirectory, listRemote.SelectedItems[i].Text, true, Int32.Parse(txtFilter.Text), Int32.Parse(txtFilter2.Text));
                        else if (optDate.Checked == true)
                            _ftp.DownloadDirectoryAsync(CurrentLocalDirectory, listRemote.SelectedItems[i].Text, true, DateTime.Parse(txtFilter.Text), DateTime.Parse(txtFilter2.Text));

                        if (!statusDialog.IsDisposed)
                            statusDialog.ShowDialog();
                        _ftp.OverallProgressEvent -= new FTP.TransferProgressChangedEventHandler(statusDialog.ftp_ProgressEvent);
                        _ftp.TransferCompleteEvent -= new FTP.TransferCompleteEventHandler(statusDialog.ftp_TransferCompleteEvent);
                    }
                }
                loadLocalDirectory(string.Empty);
            }
            catch (WebException ex)
            {
                MessageBox.Show("Web error:" + ex.Message);
                return;
            }
        }

        /// <summary>
        /// Syncs the current local and remote directories
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnSync_Click(object sender, EventArgs e)
        {
            ProgressForm statusDialog = new ProgressForm();
            try
            {
                statusDialog._ftp = _ftp;
                _ftp.OverallProgressEvent += new FTP.TransferProgressChangedEventHandler(statusDialog.ftp_ProgressEvent);
                _ftp.TransferCompleteEvent += new FTP.TransferCompleteEventHandler(statusDialog.ftp_TransferCompleteEvent);

                string oldDir = _ftp.CurrentDirectory;

                _ftp.SynchronizeDownload(CurrentLocalDirectory,string.Empty);

                if (!statusDialog.IsDisposed)
                    statusDialog.ShowDialog();
                _ftp.OverallProgressEvent -= new FTP.TransferProgressChangedEventHandler(statusDialog.ftp_ProgressEvent);
                _ftp.TransferCompleteEvent -= new FTP.TransferCompleteEventHandler(statusDialog.ftp_TransferCompleteEvent);

                _ftp.CurrentDirectory = oldDir;

                statusDialog = new ProgressForm();
                statusDialog._ftp = _ftp;
                _ftp.FileProgressEvent += new FTP.TransferProgressChangedEventHandler(statusDialog.ftp_ProgressEvent);
                _ftp.TransferCompleteEvent += new FTP.TransferCompleteEventHandler(statusDialog.ftp_TransferCompleteEvent);

                _ftp.SynchronizeUpload(CurrentLocalDirectory, string.Empty);

                if (!statusDialog.IsDisposed)
                    statusDialog.ShowDialog();
                _ftp.FileProgressEvent -= new FTP.TransferProgressChangedEventHandler(statusDialog.ftp_ProgressEvent);
                _ftp.TransferCompleteEvent -= new FTP.TransferCompleteEventHandler(statusDialog.ftp_TransferCompleteEvent);
            }
            catch (WebException ex)
            {
                MessageBox.Show("Web error:" + ex.Message);
                return;
            }
            loadLocalDirectory(string.Empty);
            loadRemoteDirectory(string.Empty);
        }

        /// <summary>
        /// Changes the local directories current drive
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cmboDrive_SelectedIndexChanged(object sender, EventArgs e)
        {
            CurrentLocalDirectory = cmboDrive.Text;
            loadLocalDirectory(string.Empty);
        }

        /// <summary>
        /// Loads a remote previously viewed remote directory
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cmboRemotePath_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (_Loading == false)
            {
                _ftp.CurrentDirectory = "";
                loadRemoteDirectory((string)cmboRemotePath.SelectedItem);
            }
        }

        /// <summary>
        /// Displays the file size of the selected remote file
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnFileSize_Click(object sender, EventArgs e)
        {
            bool shown = false;
            try
            {
                for (int i = 0; i < listRemote.SelectedItems.Count; i++)
                {
                    if (listRemote.SelectedItems[i].ImageIndex == 1)
                        continue;

                    if (listRemote.SelectedItems[i].ImageIndex == -1)
                    {
                        MessageBox.Show(_ftp.GetFileSize(listRemote.SelectedItems[i].Text).ToString());
                        shown = true;
                    }
                }
            }
            catch (WebException ex)
            {
                MessageBox.Show("Web error:" + ex.Message);
                return;
            }

            if (!shown)
                MessageBox.Show("Please select a file!");
        }

        /// <summary>
        /// Displays the directory size of the selected remote directory
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnDirSize_Click(object sender, EventArgs e)
        {
            bool shown = false;
            this.Cursor = Cursors.WaitCursor; 
            try
            {
                for (int i = 0; i < listRemote.SelectedItems.Count; i++)
                {
                    if (listRemote.SelectedItems[i].ImageIndex == 1)
                        continue;

                    if (listRemote.SelectedItems[i].ImageIndex == 0)
                    {
                        MessageBox.Show(_ftp.GetDirectorySize(listRemote.SelectedItems[i].Text).ToString());
                        shown = true;
                    }
                }

                if (!shown)
                    MessageBox.Show("Please select a directory!");
            }
            catch (WebException ex)
            {
                MessageBox.Show("Web error:" + ex.Message);
            }

            this.Cursor = Cursors.Default; 

        }

        /// <summary>
        /// Displays the file modification date of the selected remote file
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnFileDate_Click(object sender, EventArgs e)
        {
            bool shown = false;
            try
            {
                for (int i = 0; i < listRemote.SelectedItems.Count; i++)
                {
                    if (listRemote.SelectedItems[i].ImageIndex == 1)
                        continue;

                    if (listRemote.SelectedItems[i].ImageIndex == -1)
                    {
                        MessageBox.Show(_ftp.GetFileDate(listRemote.SelectedItems[i].Text).ToString());
                        shown = true;
                    }
                }
            }
            catch (WebException ex)
            {
                MessageBox.Show("Web error:" + ex.Message);
                return;
            }
            if (!shown)
                MessageBox.Show("Please select a file!");
        }

        /// <summary>
        /// Update the file with a unique name
        /// The name it is saved as will be displayed in a message box
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnUploadUnique_Click(object sender, EventArgs e)
        {
            if (listLocal.SelectedItems.Count == 0 || listLocal.SelectedItems[0].Text == "..")
                return;

            this.Cursor = Cursors.WaitCursor; 

            try
            {
                String name = _ftp.UploadFileUnique(CurrentLocalDirectory + listLocal.SelectedItems[0].Text, listLocal.SelectedItems[0].Text);                
                loadRemoteDirectory(string.Empty);
                MessageBox.Show("Uploaded file as:" + name);
            }           
            catch (WebException ex)
            {
                MessageBox.Show("Web error:" + ex.Message);
            }

            this.Cursor = Cursors.Default; 
        }

        /// <summary>
        /// Upload a file/directory syncronously
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnSyncUpload_Click(object sender, EventArgs e)
        {
            this.Cursor = Cursors.WaitCursor; 
            try
            {

                for (int i = 0; i < listLocal.SelectedItems.Count; i++)
                {
                    if (listLocal.SelectedItems[i].ImageIndex == 1)
                        continue;

                    if (listLocal.SelectedItems[i].ImageIndex == -1)
                    {
                        if (_ftp.FileExists(listLocal.SelectedItems[i].Text))
                        {
                            MessageBox.Show("File already exists!");
                            return;
                        }
                        _ftp.UploadFile(CurrentLocalDirectory + listLocal.SelectedItems[i].Text, _ftp.CurrentDirectory + "/" + listLocal.SelectedItems[i].Text);
                        loadRemoteDirectory(string.Empty);
                    }
                    else
                    {
                        _ftp.UploadDirectory(CurrentLocalDirectory + listLocal.SelectedItems[i].Text, listLocal.SelectedItems[i].Text);
                        loadRemoteDirectory(string.Empty);
                    }
                }
            }
            catch (WebException ex)
            {
                MessageBox.Show("Web error:" + ex.Message);
                return;
            }
            this.Cursor = Cursors.Default; 
        }

        /// <summary>
        /// Delete all .txt files in the current directory
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnDeleteWildcard_Click(object sender, EventArgs e)
        {
            try
            {
                _ftp.DeleteByWildcard("*.txt", false);
            }
            catch (WebException ex)
            {
                MessageBox.Show("Web error:" + ex.Message);
                return;
            }
            loadRemoteDirectory(string.Empty);
        }

        /// <summary>
        /// Send a raw command - NLST and display the response in a message box
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnCustomCmd_Click(object sender, EventArgs e)
        {
            try
            {
                MessageBox.Show(_ftp.SendRawFTP("SYST"));
            }
            catch (WebException ex)
            {
                MessageBox.Show("Web error:" + ex.Message);
                return;
            }
        }

        private void btnDownloadSync_Click(object sender, EventArgs e)
        {
            this.Cursor = Cursors.WaitCursor;
            try
            {
                for (int i = 0; i < listRemote.SelectedItems.Count; i++)
                {
                    if (listRemote.SelectedItems[i].ImageIndex == 1)
                        continue;

                    if (listRemote.SelectedItems[i].ImageIndex == -1)
                    {
                        _ftp.DownloadFile(CurrentLocalDirectory + listRemote.SelectedItems[i].Text, _ftp.CurrentDirectory + "/" + listRemote.SelectedItems[i].Text);
                        loadLocalDirectory(string.Empty);
                    }
                    else
                    {
                        _ftp.DownloadDirectory(CurrentLocalDirectory, listRemote.SelectedItems[i].Text);
                        loadLocalDirectory(string.Empty);
                    }
                }
            }
            catch (WebException ex)
            {
                MessageBox.Show("Web error:" + ex.Message);
            }
            this.Cursor = Cursors.Default; 
        }

        /// <summary>
        /// Save a key to the registry
        /// </summary>
        /// <param name="sCompanyName">Company Name</param>
        /// <param name="sProgramName">Program Name</param>
        /// <param name="sValue">Value to set</param>
        /// <param name="sSetting">The setting of the value</param>
        public static void SaveReg(string sCompanyName, string sProgramName, string sValue, string sSetting)
        {
            string sKey = string.Empty;
            Microsoft.Win32.RegistryKey oKey;

            sKey = "SOFTWARE\\" + sCompanyName + "\\" + sProgramName;
            oKey = Microsoft.Win32.Registry.CurrentUser.CreateSubKey(sKey);
            oKey.SetValue(sValue, sSetting);
        }

        /// <summary>
        /// Get a registry key.
        /// </summary>
        /// <param name="sCompanyName">Company Name</param>
        /// <param name="sProgramName">Program Name</param>
        /// <param name="sValue">Registry value to look for.</param>
        /// <returns>Registry key</returns>
        public static String GetReg(string sCompanyName, string sProgramName, string sValue)
        {
            string sKey = string.Empty;
            string sResult = string.Empty;
            Microsoft.Win32.RegistryKey oKey;

            sKey = "SOFTWARE\\" + sCompanyName + "\\" + sProgramName;

            oKey = Microsoft.Win32.Registry.CurrentUser.OpenSubKey(sKey, false);

            if (oKey != null)
                sResult = oKey.GetValue(sValue, "").ToString();

            return sResult;

        }

        private void TestTransfer_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (_ftp != null)
            {
                _ftp.Disconnect();
                SaveReg("KellermanSoftware", "Kellerman FTP", "Host", txtHost.Text);
                SaveReg("KellermanSoftware", "Kellerman FTP", "Login", txtLogin.Text);
            }
        }

        private void RecurseDirectories(FTP kFTP, string dir)
        {
            kFTP.CurrentDirectory = dir;
            List<FTPFileInfo> ftpFiles= kFTP.GetDirectoryListing();

            foreach (FTPFileInfo file in ftpFiles)
            {
                if (file.IsDirectory)
                {
                    Console.WriteLine("D: " + dir + file.FileName + "/");

                    if (file.FileName == "." || file.FileName == "..")
                    {
                    }
                    else
                    {
                        RecurseDirectories(kFTP, dir + file.FileName + "/");
                    }
                }
                else
                {
                    Console.WriteLine("F: " + file.FileName);
                }
            }
        }










    }
}