using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.IO;
using KellermanSoftware.NetFtpLibrary;


namespace NetFtpLibrarySimpleDemo
{
    public partial class Demo : Form
    {
        FTP _ftp;
        List<FTPFileInfo> files;
        delegate void setStatusDelegate(string text);
        delegate void loadDirectoryDelegate(string path);
        delegate void setCommandGUIDelegate(bool enabled);

        public Demo()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Show an error message box
        /// </summary>
        /// <param name="myException"></param>
        private void ShowError(Exception myException)
        {            
            MessageBox.Show(myException.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }

        /// <summary>
        /// Event that is fired when a transfer completes
        /// </summary>
        private void ftp_TransferCompleted(Object sender, TransferCompletedEventArgs e)
        {
            setStatus("File transfer completed...");
            loadDirectory(string.Empty);
            setCommandGUI(true);
        }

        /// <summary>
        /// Event that is fired when a transfer completes
        /// </summary>
        private void ftp_TransferProgressChangedEvent(Object sender, TransferProgressChangedEventArgs e)
        {
            try
            { 
                setStatus("Transfer [" + _ftp.CurrentFileBeingTransferred + " in " + _ftp.CurrentDirectoryBeingTransferred + "]: " + e.ProgressPercentage + "% (" + e.BytesTransferred + " of " + e.TotalBytesToTransfer + " bytes)");
            }
            catch (Exception myException)
            {
                ShowError(myException);
                Application.Exit();
            }
        }

        /// <summary>
        /// Fired when form loads: initializes the FTPTransfer class and sets the event handlers
        /// </summary>
        private void Form1_Load(object sender, EventArgs e)
        {
            try
            { 
                _ftp = new FTP();
                _ftp.OverallProgressEvent += new FTP.TransferProgressChangedEventHandler(ftp_TransferProgressChangedEvent);
                _ftp.FileProgressEvent += new FTP.TransferProgressChangedEventHandler(ftp_TransferProgressChangedEvent);
                _ftp.TransferCompleteEvent += new FTP.TransferCompleteEventHandler(ftp_TransferCompleted);
                txtGMT.Text = _ftp.HourDifference.ToString();
            }
            catch (Exception myException)
            {
                ShowError(myException);
                Application.Exit();
            }
        }

        /// <summary>
        /// Either enables or disables the connection GUI
        /// </summary>
        private void setConnectGUI(bool enabled)
        {
            txtHost.Enabled = enabled;
            txtLogin.Enabled = enabled;
            txtPass.Enabled = enabled;
            txtPort.Enabled = enabled;
            txtGMT.Enabled = enabled;
            chkBinary.Enabled = enabled;
            chkOverWrite.Enabled = enabled;
            chkPassive.Enabled = enabled;
            chkReadOnly.Enabled = enabled;
            chkSSL.Enabled = enabled;
        }

        /// <summary>
        /// Either enables or disables the FTP command GUI
        /// </summary>
        private void setCommandGUI(bool enabled)
        {
            // this gets called from an event, so make sure this is on the right thread
            if (this.InvokeRequired)
            {
                // this is not on the right thread, so invoke it on the correct one
                setCommandGUIDelegate setCommandGUIProc = new setCommandGUIDelegate(setCommandGUI);
                this.Invoke(setCommandGUIProc, new object[] { enabled });
                return;
            }

            btnCreateDirectory.Enabled = enabled;
            btnDeleteDirectory.Enabled = enabled;
            btnDownloadDir.Enabled = enabled;
            btnUploadDir.Enabled = enabled;

            btnDownloadFile.Enabled = enabled;
            btnUploadFile.Enabled = enabled;
            btnDeleteFile.Enabled = enabled;
            btnRenameFile.Enabled = enabled;
        }

        /// <summary>
        /// Set the status bar text
        /// </summary>
        private void setStatus(String s)
        {
            // this gets called from an event, so make sure this is on the right thread
            if (status.InvokeRequired)
            {
                // this is not on the right thread, so invoke it on the correct one
                setStatusDelegate setStatusProc = new setStatusDelegate(setStatus);
                status.Invoke(setStatusProc, new object[] { s });
                return;
            }
            status.Items["Label"].Text = s;
            status.Refresh();
        }

        /// <summary>
        /// Set the status bar text to the status code of an FTP Request
        /// </summary>
        private void SetCode(FtpResponse response)
        {
            status.Items["FTPCode"].Text = "FTP Status: " + response.Text;
            status.Refresh();
        }

        /// <summary>
        /// Load a directory listing into the GUI
        /// </summary>
        /// <param name="dirName">The directory to load, pass string.empty to reload the current directory</param>
        private void loadDirectory(String dirName)
        {
            try
            { 
                setStatus("Loading directory listing...");

                // this gets called from an event, so make sure this is on the right thread
                if (treeDir.InvokeRequired)
                {
                    // this is not on the right thread, so invoke it on the correct one
                    loadDirectoryDelegate loadDirectoryProc = new loadDirectoryDelegate(loadDirectory);
                    this.Invoke(loadDirectoryProc, new object[] { dirName });
                    return;
                }


                if (dirName.Length > 0)
                    _ftp.CurrentDirectory = _ftp.CurrentDirectory + "/" + dirName;

                files = _ftp.GetDirectoryListing();
                TreeNode n;

                treeDir.Nodes.Clear();
                treeFiles.Nodes.Clear();
                bool upDir = false;

                foreach (FTPFileInfo file in files)
                {
                    if (file.IsDirectory)
                    {
                        n = treeDir.Nodes.Add(file.FileName);
                        if (file.FileName == ".." || file.FileName == ".")
                            upDir = true;
                    }
                    else
                    {
                        n = treeFiles.Nodes.Add(file.FileName);
                        n.ToolTipText = "Modified: " + file.Modified.ToString() + "\nSize: " + file.SizeInBytes;
                    }
                }

                if(!upDir)
                    treeDir.Nodes.Insert(0, "..");

                setStatus("");
            }
            catch (Exception myException)
            {
                ShowError(myException);
                Application.Exit();
            }
        }

        /// <summary>
        /// Event that is fired when a the connect button is clicked, set the FTPTransfer properties and load the default directory
        /// </summary>
        private void btnConnect_Click(object sender, EventArgs e)
        {
            try
            { 
                btnConnect.Enabled = false;
                if (btnConnect.Text == "Go")
                {
                    setStatus("Logging in...");
                    setConnectGUI(false);
                    _ftp.HostAddress = txtHost.Text;
                    _ftp.UserName = txtLogin.Text;
                    _ftp.Password = txtPass.Text;
                    _ftp.Port = Int32.Parse(txtPort.Text);
                    _ftp.UseBinary = chkBinary.Checked;
                    _ftp.OverwriteFiles = chkOverWrite.Checked;
                    _ftp.OverwriteReadOnlyFiles = chkReadOnly.Checked;
                    _ftp.EnableSSL = chkSSL.Checked;
                    _ftp.UsePassive = chkPassive.Checked;
                    _ftp.Connect();

                    loadDirectory(string.Empty);
                    btnConnect.Text = "Dispose";
                    setCommandGUI(true);
                }
                else
                {
                    _ftp.Disconnect();
                    _ftp.Dispose();
                    treeDir.Nodes.Clear();
                    treeFiles.Nodes.Clear();
                    btnConnect.Text = "Go";
                    setConnectGUI(true);
                    setCommandGUI(false);
                }
                btnConnect.Enabled = true;
            }
            catch (Exception myException)
            {
                ShowError(myException);
                Application.Exit();
            }
        }

        /// <summary>
        /// Delete a selected directory
        /// </summary>
        private void btnDeleteDirectory_Click(object sender, EventArgs e)
        {
            try
            { 
                if (treeDir.SelectedNode != null)
                {
                    if (treeDir.SelectedNode.Text != "." && treeDir.SelectedNode.Text != "..")
                    {
                        setStatus("Deleting directory...");
                        if (_ftp.DeleteDirectory(treeDir.SelectedNode.Text))
                        {
                            setStatus("");
                            loadDirectory(string.Empty);
                            SetCode(_ftp.LastFtpResponse);
                        }
                        else
                            setStatus("Directory deletion failed...");
                    }
                }
                else
                    MessageBox.Show("Please select a directory to delete!");
            }
            catch (Exception myException)
            {
                ShowError(myException);
                Application.Exit();
            }
        }

        /// <summary>
        /// Delete a selected file
        /// </summary>
        private void btnDeleteFile_Click(object sender, EventArgs e)
        {
            try
            { 
                if (treeFiles.SelectedNode != null)
                {
                    setStatus("Deleting file...");
                    _ftp.DeleteFile(treeFiles.SelectedNode.Text);
                    loadDirectory(string.Empty);
                    SetCode(_ftp.LastFtpResponse);
                }
                else
                    MessageBox.Show("Please select a file to delete!");
            }
            catch (Exception myException)
            {
                ShowError(myException);
                Application.Exit();
            }
        }

        /// <summary>
        /// Fired when the directory tree is double clicked, navigate to that directory
        /// </summary>
        private void treeDir_DoubleClick(object sender, EventArgs e)
        {
            if(treeDir.SelectedNode != null) loadDirectory(treeDir.SelectedNode.Text);
        }

        /// <summary>
        /// Rename a selected file by placing an edit box on it
        /// </summary>
        private void btnRenameFile_Click(object sender, EventArgs e)
        {
            if (treeFiles.SelectedNode == null)
            {
                MessageBox.Show("Please select a file to rename!");
                return;
            }

            treeFiles.LabelEdit = true;
            if (treeFiles.SelectedNode.IsEditing == false)
                treeFiles.SelectedNode.BeginEdit();

        }

        /// <summary>
        /// Fired when the user is done editing the file's name, update it
        /// </summary>
        private void treeFiles_AfterLabelEdit(object sender, System.Windows.Forms.NodeLabelEditEventArgs e)
        {
            try
            {
                if (e.Label != null)
                {
                    if(e.Label.Length > 0)
                    {
                        setStatus("Checking for file existence...");
                        if (_ftp.FileExists(e.Label))
                        {
                            MessageBox.Show("File already exists!");
                            e.CancelEdit = true;
                        }
                        else
                        {
                            setStatus("Renaming file...");
                            _ftp.RenameFile(e.Node.Text, e.Label);
                            SetCode(_ftp.LastFtpResponse);
                        }
                                        
                        setStatus("");
                    }
                    else
                    {
                        MessageBox.Show("File name's cannot be blank!");
                        e.CancelEdit = true;
                    }
                }
                this.treeFiles.LabelEdit = false;
            }
            catch (Exception myException)
            {
                ShowError(myException);
                Application.Exit();
            }
        }

        /// <summary>
        /// Fired when the user is done editing the new directory's name, create it
        /// </summary>
        private void treeDir_AfterLabelEdit(object sender, System.Windows.Forms.NodeLabelEditEventArgs e)
        {
            try
            { 
                if (e.Label != null)
                {
                    if (e.Label.Length > 0)
                    {
                        bool exists = false;
                        foreach (TreeNode n in treeDir.Nodes)
                            if (n.Text == e.Label && !n.IsEditing)
                            {
                                exists = true;
                                break;
                            }

                        if (exists)
                        {
                            e.CancelEdit = true;
                            e.Node.Remove();
                            MessageBox.Show("Directory already exists!");
                        }
                        else
                        {
                            setStatus("Creating directory...");
                            _ftp.CreateDirectory(e.Label);
                            SetCode(_ftp.LastFtpResponse);
                        }

                        setStatus("");
                    }
                    else
                    {
                        MessageBox.Show("Directory name cannot be blank!");
                        e.CancelEdit = true;
                    }
                }
                this.treeFiles.LabelEdit = false;
            }
            catch (Exception myException)
            {
                ShowError(myException);
                Application.Exit();
            }
        }

        /// <summary>
        /// Upload a file
        /// </summary>
        private void btnUploadFile_Click(object sender, EventArgs e)
        {
            try
            { 
                if (openFileDialog.ShowDialog() == DialogResult.OK)
                {
                    // check for file existence
                    String fileName = Path.GetFileName(openFileDialog.FileName);
                    if (!_ftp.FileExists(fileName))
                    {
                        setCommandGUI(false);
                        _ftp.UploadFileAsync(openFileDialog.FileName, fileName);
                    }
                    else
                    {
                        MessageBox.Show("File already exists!");
                    }
                }
            }
            catch (Exception myException)
            {
                ShowError(myException);
                Application.Exit();
            }
        }

        /// <summary>
        /// Download a file
        /// </summary>
        private void btnDownloadFile_Click(object sender, EventArgs e)
        {
            try
            { 
                if (treeFiles.SelectedNode == null)
                {
                    setCommandGUI(false);
                    MessageBox.Show("Please select a file to download!");
                    return;
                }

                saveFileDialog.FileName = treeFiles.SelectedNode.Text;
                if (saveFileDialog.ShowDialog() == DialogResult.OK)
                {
                    _ftp.DownloadFileAsync(saveFileDialog.FileName, treeFiles.SelectedNode.Text);
                }
            }
            catch (Exception myException)
            {
                ShowError(myException);
                Application.Exit();
            }
        }

        /// <summary>
        /// Begin creating a directory by creating a new edit box for the name
        /// </summary>
        private void btnCreateDirectory_Click(object sender, EventArgs e)
        {
            treeDir.LabelEdit = true;
            TreeNode n = treeDir.Nodes.Add("New Directory", "New Directory");
            treeDir.SelectedNode = n;
            n.BeginEdit();
        }

        /// <summary>
        /// Upload a directory
        /// </summary>
        private void btnUploadDir_Click(object sender, EventArgs e)
        {
            try
            { 
                if(folderBrowser.ShowDialog() == DialogResult.OK)
                {
                    setCommandGUI(false);
                    DirectoryInfo d = new DirectoryInfo(folderBrowser.SelectedPath);
                    _ftp.UploadDirectoryAsync(folderBrowser.SelectedPath, d.Name);
                }
            }
            catch (Exception myException)
            {
                ShowError(myException);
                Application.Exit();
            }
        }

        /// <summary>
        /// Download a directory
        /// </summary>
        private void btnDownloadDir_Click(object sender, EventArgs e)
        {
            try
            { 
                if (treeDir.SelectedNode == null)
                {
                    setCommandGUI(true);
                    MessageBox.Show("Please select a directory to download!");
                    return;
                }

                if (folderBrowser.ShowDialog() == DialogResult.OK)
                {
                    setCommandGUI(false);
                    _ftp.DownloadDirectoryAsync(folderBrowser.SelectedPath,treeDir.SelectedNode.Text);
                }
            }
            catch (Exception myException)
            {
                ShowError(myException);
                Application.Exit();
            }
        }

        /// <summary>
        /// Cancel a transfer
        /// </summary>
        private void btnCancel_Click(object sender, EventArgs e)
        {
            try
            { 
                _ftp.Cancel();
            }
            catch (Exception myException)
            {
                ShowError(myException);
                Application.Exit();
            }
        }
    }
}