﻿Imports System
Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.Data
Imports System.Drawing
Imports System.Text
Imports System.Windows.Forms
Imports System.IO
Imports KellermanSoftware.NetFtpLibrary


Namespace NetFtpLibrarySimpleDemo
	Partial Public Class Demo
		Inherits Form

		Private _ftp As FTP
		Private files As List(Of FTPFileInfo)
		Private Delegate Sub setStatusDelegate(ByVal text As String)
		Private Delegate Sub loadDirectoryDelegate(ByVal path As String)
		Private Delegate Sub setCommandGUIDelegate(ByVal enabled As Boolean)

		Public Sub New()
			InitializeComponent()
		End Sub

		''' <summary>
		''' Event that is fired when a transfer completes
		''' </summary>
		Private Sub ftp_TransferCompleted(ByVal sender As Object, ByVal e As TransferCompletedEventArgs)
			setStatus("File transfer completed...")
			loadDirectory(String.Empty)
			setCommandGUI(True)
		End Sub

		''' <summary>
		''' Event that is fired when a transfer completes
		''' </summary>
		Private Sub ftp_TransferProgressChangedEvent(ByVal sender As Object, ByVal e As TransferProgressChangedEventArgs)
			setStatus("Transfer [" & _ftp.CurrentFileBeingTransferred & " in " & _ftp.CurrentDirectoryBeingTransferred & "]: " & e.ProgressPercentage & "% (" & e.BytesTransferred & " of " & e.TotalBytesToTransfer & " bytes)")
		End Sub

		''' <summary>
		''' Fired when form loads: initializes the FTPTransfer class and sets the event handlers
		''' </summary>
		Private Sub Form1_Load(ByVal sender As Object, ByVal e As EventArgs) Handles Me.Load
			_ftp = New FTP()
			AddHandler _ftp.OverallProgressEvent, AddressOf ftp_TransferProgressChangedEvent
			AddHandler _ftp.FileProgressEvent, AddressOf ftp_TransferProgressChangedEvent
			AddHandler _ftp.TransferCompleteEvent, AddressOf ftp_TransferCompleted
			txtGMT.Text = _ftp.HourDifference.ToString()
		End Sub

		''' <summary>
		''' Either enables or disables the connection GUI
		''' </summary>
		Private Sub setConnectGUI(ByVal enabled As Boolean)
			txtHost.Enabled = enabled
			txtLogin.Enabled = enabled
			txtPass.Enabled = enabled
			txtPort.Enabled = enabled
			txtGMT.Enabled = enabled
			chkBinary.Enabled = enabled
			chkOverWrite.Enabled = enabled
			chkPassive.Enabled = enabled
			chkReadOnly.Enabled = enabled
			chkSSL.Enabled = enabled
		End Sub

		''' <summary>
		''' Either enables or disables the FTP command GUI
		''' </summary>
		Private Sub setCommandGUI(ByVal enabled As Boolean)
			' this gets called from an event, so make sure this is on the right thread
			If Me.InvokeRequired Then
				' this is not on the right thread, so invoke it on the correct one
				Dim setCommandGUIProc As New setCommandGUIDelegate(AddressOf setCommandGUI)
				Me.Invoke(setCommandGUIProc, New Object() { enabled })
				Return
			End If

			btnCreateDirectory.Enabled = enabled
			btnDeleteDirectory.Enabled = enabled
			btnDownloadDir.Enabled = enabled
			btnUploadDir.Enabled = enabled

			btnDownloadFile.Enabled = enabled
			btnUploadFile.Enabled = enabled
			btnDeleteFile.Enabled = enabled
			btnRenameFile.Enabled = enabled
		End Sub

		''' <summary>
		''' Set the status bar text
		''' </summary>
		Private Sub setStatus(ByVal s As String)
			' this gets called from an event, so make sure this is on the right thread
			If status.InvokeRequired Then
				' this is not on the right thread, so invoke it on the correct one
				Dim setStatusProc As New setStatusDelegate(AddressOf setStatus)
				status.Invoke(setStatusProc, New Object() { s })
				Return
			End If
			status.Items("Label").Text = s
			status.Refresh()
		End Sub

		''' <summary>
		''' Set the status bar text to the status code of an FTP Request
		''' </summary>
		Private Sub SetCode(ByVal response As FtpResponse)
			status.Items("FTPCode").Text = "FTP Status: " & response.Text
			status.Refresh()
		End Sub

		''' <summary>
		''' Load a directory listing into the GUI
		''' </summary>
		''' <param name="dirName">The directory to load, pass string.empty to reload the current directory</param>
		Private Sub loadDirectory(ByVal dirName As String)
			setStatus("Loading directory listing...")

			' this gets called from an event, so make sure this is on the right thread
			If treeDir.InvokeRequired Then
				' this is not on the right thread, so invoke it on the correct one
				Dim loadDirectoryProc As New loadDirectoryDelegate(AddressOf loadDirectory)
				Me.Invoke(loadDirectoryProc, New Object() { dirName })
				Return
			End If


			If dirName.Length > 0 Then
				_ftp.CurrentDirectory = _ftp.CurrentDirectory & "/" & dirName
			End If

			files = _ftp.GetDirectoryListing()
			Dim n As TreeNode

			treeDir.Nodes.Clear()
			treeFiles.Nodes.Clear()
			Dim upDir As Boolean = False

			For Each file As FTPFileInfo In files
				If file.IsDirectory Then
					n = treeDir.Nodes.Add(file.FileName)
					If file.FileName = ".." OrElse file.FileName = "." Then
						upDir = True
					End If
				Else
					n = treeFiles.Nodes.Add(file.FileName)
					n.ToolTipText = "Modified: " & file.Modified.ToString() & ControlChars.Lf & "Size: " & file.SizeInBytes
				End If
			Next file

			If Not upDir Then
				treeDir.Nodes.Insert(0, "..")
			End If

			setStatus("")
		End Sub

		''' <summary>
		''' Event that is fired when a the connect button is clicked, set the FTPTransfer properties and load the default directory
		''' </summary>
		Private Sub btnConnect_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnConnect.Click
			btnConnect.Enabled = False
			If btnConnect.Text = "Go" Then
				setStatus("Logging in...")
				setConnectGUI(False)
				_ftp.HostAddress = txtHost.Text
				_ftp.UserName = txtLogin.Text
				_ftp.Password = txtPass.Text
				_ftp.Port = Int32.Parse(txtPort.Text)
				_ftp.UseBinary = chkBinary.Checked
				_ftp.OverwriteFiles = chkOverWrite.Checked
				_ftp.OverwriteReadOnlyFiles = chkReadOnly.Checked
				_ftp.EnableSSL = chkSSL.Checked
				_ftp.UsePassive = chkPassive.Checked
				_ftp.Connect()

				loadDirectory(String.Empty)
				btnConnect.Text = "Dispose"
				setCommandGUI(True)
			Else
				_ftp.Disconnect()
				_ftp.Dispose()
				treeDir.Nodes.Clear()
				treeFiles.Nodes.Clear()
				btnConnect.Text = "Go"
				setConnectGUI(True)
				setCommandGUI(False)
			End If
			btnConnect.Enabled = True
		End Sub

		''' <summary>
		''' Delete a selected directory
		''' </summary>
		Private Sub btnDeleteDirectory_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnDeleteDirectory.Click
			If treeDir.SelectedNode IsNot Nothing Then
				If treeDir.SelectedNode.Text <> "." AndAlso treeDir.SelectedNode.Text <> ".." Then
					setStatus("Deleting directory...")
					If _ftp.DeleteDirectory(treeDir.SelectedNode.Text) Then
						setStatus("")
						loadDirectory(String.Empty)
						SetCode(_ftp.LastFtpResponse)
					Else
						setStatus("Directory deletion failed...")
					End If
				End If
			Else
				MessageBox.Show("Please select a directory to delete!")
			End If
		End Sub

		''' <summary>
		''' Delete a selected file
		''' </summary>
		Private Sub btnDeleteFile_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnDeleteFile.Click
			If treeFiles.SelectedNode IsNot Nothing Then
				setStatus("Deleting file...")
				_ftp.DeleteFile(treeFiles.SelectedNode.Text)
				loadDirectory(String.Empty)
				SetCode(_ftp.LastFtpResponse)
			Else
				MessageBox.Show("Please select a file to delete!")
			End If
		End Sub

		''' <summary>
		''' Fired when the directory tree is double clicked, navigate to that directory
		''' </summary>
		Private Sub treeDir_DoubleClick(ByVal sender As Object, ByVal e As EventArgs) Handles treeDir.DoubleClick
			If treeDir.SelectedNode IsNot Nothing Then
				loadDirectory(treeDir.SelectedNode.Text)
			End If
		End Sub

		''' <summary>
		''' Rename a selected file by placing an edit box on it
		''' </summary>
		Private Sub btnRenameFile_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnRenameFile.Click
			If treeFiles.SelectedNode Is Nothing Then
				MessageBox.Show("Please select a file to rename!")
				Return
			End If

			treeFiles.LabelEdit = True
			If treeFiles.SelectedNode.IsEditing = False Then
				treeFiles.SelectedNode.BeginEdit()
			End If

		End Sub

		''' <summary>
		''' Fired when the user is done editing the file's name, update it
		''' </summary>
		Private Sub treeFiles_AfterLabelEdit(ByVal sender As Object, ByVal e As System.Windows.Forms.NodeLabelEditEventArgs) Handles treeFiles.AfterLabelEdit
			If e.Label IsNot Nothing Then
				If e.Label.Length > 0 Then
					setStatus("Checking for file existence...")
					If _ftp.FileExists(e.Label) Then
						MessageBox.Show("File already exists!")
						e.CancelEdit = True
					Else
						setStatus("Renaming file...")
						_ftp.RenameFile(e.Node.Text, e.Label)
						SetCode(_ftp.LastFtpResponse)
					End If

					setStatus("")
				Else
					MessageBox.Show("File name's cannot be blank!")
					e.CancelEdit = True
				End If
			End If
			Me.treeFiles.LabelEdit = False
		End Sub

		''' <summary>
		''' Fired when the user is done editing the new directory's name, create it
		''' </summary>
		Private Sub treeDir_AfterLabelEdit(ByVal sender As Object, ByVal e As System.Windows.Forms.NodeLabelEditEventArgs) Handles treeDir.AfterLabelEdit
			If e.Label IsNot Nothing Then
				If e.Label.Length > 0 Then
					Dim exists As Boolean = False
					For Each n As TreeNode In treeDir.Nodes
						If n.Text = e.Label AndAlso Not n.IsEditing Then
							exists = True
							Exit For
						End If
					Next n

					If exists Then
						e.CancelEdit = True
						e.Node.Remove()
						MessageBox.Show("Directory already exists!")
					Else
						setStatus("Creating directory...")
						_ftp.CreateDirectory(e.Label)
						SetCode(_ftp.LastFtpResponse)
					End If

					setStatus("")
				Else
					MessageBox.Show("Directory name cannot be blank!")
					e.CancelEdit = True
				End If
			End If
			Me.treeFiles.LabelEdit = False
		End Sub

		''' <summary>
		''' Upload a file
		''' </summary>
		Private Sub btnUploadFile_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnUploadFile.Click
			If openFileDialog.ShowDialog() = System.Windows.Forms.DialogResult.OK Then
				' check for file existence
				Dim fileName As String = Path.GetFileName(openFileDialog.FileName)
				If Not _ftp.FileExists(fileName) Then
					setCommandGUI(False)
					_ftp.UploadFileAsync(openFileDialog.FileName, fileName)
				Else
					MessageBox.Show("File already exists!")
				End If
			End If
		End Sub

		''' <summary>
		''' Download a file
		''' </summary>
		Private Sub btnDownloadFile_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnDownloadFile.Click
			If treeFiles.SelectedNode Is Nothing Then
				setCommandGUI(False)
				MessageBox.Show("Please select a file to download!")
				Return
			End If

			saveFileDialog.FileName = treeFiles.SelectedNode.Text
			If saveFileDialog.ShowDialog() = System.Windows.Forms.DialogResult.OK Then
				_ftp.DownloadFileAsync(saveFileDialog.FileName, treeFiles.SelectedNode.Text)
			End If
		End Sub

		''' <summary>
		''' Begin creating a directory by creating a new edit box for the name
		''' </summary>
		Private Sub btnCreateDirectory_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnCreateDirectory.Click
			treeDir.LabelEdit = True
			Dim n As TreeNode = treeDir.Nodes.Add("New Directory", "New Directory")
			treeDir.SelectedNode = n
			n.BeginEdit()
		End Sub

		''' <summary>
		''' Upload a directory
		''' </summary>
		Private Sub btnUploadDir_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnUploadDir.Click
			If folderBrowser.ShowDialog() = System.Windows.Forms.DialogResult.OK Then
				setCommandGUI(False)
				Dim d As New DirectoryInfo(folderBrowser.SelectedPath)
				_ftp.UploadDirectoryAsync(folderBrowser.SelectedPath, d.Name)
			End If
		End Sub

		''' <summary>
		''' Download a directory
		''' </summary>
		Private Sub btnDownloadDir_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnDownloadDir.Click
			If treeDir.SelectedNode Is Nothing Then
				setCommandGUI(True)
				MessageBox.Show("Please select a directory to download!")
				Return
			End If

			If folderBrowser.ShowDialog() = System.Windows.Forms.DialogResult.OK Then
				setCommandGUI(False)
				_ftp.DownloadDirectoryAsync(folderBrowser.SelectedPath,treeDir.SelectedNode.Text)
			End If
		End Sub

		''' <summary>
		''' Cancel a transfer
		''' </summary>
		Private Sub btnCancel_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnCancel.Click
			_ftp.Cancel()
		End Sub


	End Class
End Namespace